import ast
import inspect
import re
import os
import sys

from django import forms
from django.conf import settings
from django.core.exceptions import ValidationError
from django.core.validators import MinValueValidator, MaxValueValidator
from django.forms import formset_factory
from django.forms import ModelChoiceField
from django.forms.fields import *
from django.template.defaultfilters import filesizeformat
from django.utils.module_loading import import_string
from django.utils.translation import gettext as _

from filesig.filesig import get_signatures

from . settings import *
from . utils import (_split_choices,
                     _split_choices_in_list_canc,
                     _successivo_ad_oggi)

def get_fields_types(class_name=sys.modules[__name__]):
    fields_types = []
    for m in inspect.getmembers(class_name, inspect.isclass):
        if hasattr(m[1],'field_type'):
            field_type = getattr(m[1], 'field_type')
            if field_type:
                fields_types.append(tuple((m[1].__name__, field_type)))
    fields_types.sort(key=lambda tup: tup[1])
    return fields_types

def format_field_name(field_name, lower=True):
    f = field_name.replace(' ','_')
    f = f.replace('(','')
    f = f.replace(')','')
    f = f.replace('/','_')
    if lower: return f.lower()
    return f.upper

def get_empty_form(form_class=forms.Form):
    class Dynamic(form_class):
        pass
    return Dynamic

def build_formset(choices, extra=0, required=False, prefix='form', data=None):
    """ Get formset
    """
    _regexp = '(?P<colname>[a-zA-Z0-9_]*)\((?P<coldict>[\{\}\.0-9a-zA-Z\'\"\:\;\_\,\s\-]*)\)'
    min_num = 0
    if required: min_num = 1
    eform = get_empty_form()
    for choice in choices:
        colname = choice # needed for simple CharField withoud attrs
        contenuto = re.search(_regexp, choice)
        field_dict = None
        if contenuto:
            coldict = contenuto.groupdict().get('coldict')
            colname = contenuto.groupdict()['colname']
            if coldict:
                field_dict = ast.literal_eval(coldict)
                field_type_name = field_dict['type']
                del field_dict['type']
                custom_field = getattr(sys.modules[__name__], field_type_name)(**field_dict) \
                               if hasattr(sys.modules[__name__], field_type_name) else CustomCharField()
                custom_widget = getattr(settings, 'CUSTOM_WIDGETS').get(field_type_name) \
                                if hasattr(settings, 'CUSTOM_WIDGETS') else None
                if custom_widget:
                    custom_field.widget = import_string(custom_widget)()

                if field_dict.get('choices'):
                    custom_field.choices += _split_choices(field_dict.get('choices'))
        else:
            custom_field = CustomCharField()
        eform.base_fields[colname] = custom_field
        eform.declared_fields[colname] = custom_field

    # Django formset
    fac =  formset_factory(eform, extra=extra, min_num=min_num)
    if data: return fac(prefix=prefix, data=data)
    return fac(prefix=prefix)


class BaseCustomField(Field):
    """
    Classe Base che definisce i metodi per ogni CustomField
    """
    is_complex = False
    is_formset = False

    def define_value(self, custom_value=None, **kwargs):
        """
        Integra la costruzione del field con informazioni aggiuntive
        provenienti dai parametri di configurazione definiti dall'utente
        """
        return

    def get_fields(self):
        """
        Se è un field semplice, torna se stesso.
        Se è un field composto, torna una lista di fields.
        """
        return [self]

    def raise_error(self, name, cleaned_data, **kwargs):
        """
        Torna la lista degli errori generati dalla validazione del field
        """
        return []


class CustomCharField(CharField, BaseCustomField):
    """
    CharField
    """
    field_type = _("Testo")

    def __init__(self, *args, **data_kwargs):
        super().__init__(*args, **data_kwargs)


class CustomEmailField(EmailField, BaseCustomField):
    """
    EmailField
    """
    field_type = _("E-mail")


class CustomChoiceField(ChoiceField, BaseCustomField):
    """
    ChoiceField
    """
    def __init__(self, *args, **data_kwargs):
        super().__init__(*args, **data_kwargs)
        self.choices = []

    def define_value(self, choices, **kwargs):
        """
        Se presenti, sostituisce alle opzioni di default
        quelle di 'custom_value'
        """
        # Imposta le 'choices' definite in backend come opzioni
        if choices:
            self.choices += _split_choices(choices)


class CustomFileField(FileField, BaseCustomField):
    """
    FileField
    """
    field_type = _("Allegato PDF")

    def __init__(self, *args, **data_kwargs):
        super().__init__(*args, **data_kwargs)

    def raise_error(self, name, cleaned_data, **kwargs):
        data = cleaned_data
        errors = []
        if data:
            msg = ''
            permitted_upload_filetype = getattr(settings, 'PERMITTED_UPLOAD_FILETYPE') if hasattr(settings, 'PERMITTED_UPLOAD_FILETYPE') else PERMITTED_UPLOAD_FILETYPE
            max_upload_size = getattr(settings, 'MAX_UPLOAD_SIZE') if hasattr(settings, 'MAX_UPLOAD_SIZE') else MAX_UPLOAD_SIZE
            attach_max_len = getattr(settings, 'ATTACH_NAME_MAX_LEN') if hasattr(settings, 'ATTACH_NAME_MAX_LEN') else ATTACH_NAME_MAX_LEN

            if data.content_type not in permitted_upload_filetype:
                msg_tmpl = WRONG_TYPE
                msg = msg_tmpl.format(permitted_upload_filetype, data.content_type)
            elif data.size > int(max_upload_size):
                msg_tmpl = WRONG_SIZE
                msg = msg_tmpl.format(filesizeformat(max_upload_size),
                                      filesizeformat(data.size))
            elif len(data._name) > attach_max_len:
                msg_tmpl = WRONG_LENGTH
                msg = msg_tmpl.format(attach_max_len, len(data._name))
            if msg: errors.append(msg)
        return errors


class CustomSignedFileField(CustomFileField):
    validation_error = _('Errore di validazione della firma digitale')
    fileformat = ''
    field_type = None

    def get_signature_params(self, data):
        """
        """
        if not data: return
        res = get_signatures(data, type=self.fileformat, only_valids=True)
        return res

    def get_cleaned_signature_params(self, data):
        res = self.get_signature_params(data)
        details = {}
        details['Signature Validation'] = res[-1].get('Signature Validation')
        details['Signing Time'] = res[-1].get('Signing Time')
        details['Signer full Distinguished Name'] = res[-1].get('Signer full Distinguished Name')
        return details

    def raise_error(self, name, cleaned_data, **kwargs):
        data = cleaned_data
        errors = []
        if data:
            simple_filefield_errors = super().raise_error(name, cleaned_data, **kwargs)
            errors.extend(simple_filefield_errors)
            res = self.get_signature_params(data)
            if not res or 'is valid' not in res[-1].get('Signature Validation').lower():
                errors.append(self.validation_error)
        return errors


class CustomSignedPdfField(CustomSignedFileField):
    """
    """
    field_type = _("Allegato PDF firmato")
    fileformat = 'pdf'


class CustomSignedP7MField(CustomSignedFileField):
    """
    """
    field_type = _("Allegato P7M firmato")
    fileformat = 'p7m'


class PositiveIntegerField(DecimalField, BaseCustomField):
    """
    Int DecimalField positivo
    """
    field_type = _("Numero intero positivo")
    default_validators = [MinValueValidator(0)]

    def __init__(self, *args, **data_kwargs):
        # Non si accettano formati con cifre decimali
        data_kwargs['decimal_places'] = 0
        super().__init__(*args, **data_kwargs)

    def raise_error(self, name, cleaned_data, **kwargs):
        """
        Solo numeri (espressioni del tipo 16e50 non sono ammesse)
        """
        if not cleaned_data: return []
        if not re.match('^[0-9]+$', str(cleaned_data)):
            return [_("Solo numeri ammessi"),]


class PositiveFloatField(DecimalField, BaseCustomField):
    """
    Float DecimalField positivo
    """
    field_type = _("Numero con virgola positivo")
    default_validators = [MinValueValidator(0)]

    def __init__(self, *args, **data_kwargs):
        # Max 3 cifre decimali
        data_kwargs['decimal_places'] = 3
        super().__init__(*args, **data_kwargs)

    def raise_error(self, name, cleaned_data, **kwargs):
        """
        Solo numeri (espressioni del tipo 16e50 non sono ammesse)
        """
        if not cleaned_data: return []
        if not re.match('^[0-9]+\.?[0-9]?$', str(cleaned_data)):
            return [_("Solo numeri ammessi"),]


class TextAreaField(CharField, BaseCustomField):
    """
    TextArea
    """
    field_type = _("Testo lungo")
    widget = forms.Textarea()


class CheckBoxField(BooleanField, BaseCustomField):
    """
    BooleanField Checkbox
    """
    field_type = _("Checkbox")
    widget = forms.CheckboxInput()


class CustomSelectBoxField(CustomChoiceField):
    """
    SelectBox
    """
    field_type = _("Lista di opzioni (tendina)")

    def __init__(self, *args, **data_kwargs):
        super().__init__(*args, **data_kwargs)
        self.choices = [('', _('Scegli una opzione')),]


class CustomRadioBoxField(CustomChoiceField):
    """
    CheckBox multiplo
    """
    field_type = _("Lista di opzioni (checkbox)")
    widget = forms.RadioSelect()

class BaseDateField(DateField, BaseCustomField):
    """
    DateField
    """
    field_type = _("Data")
    widget = forms.DateInput()
    input_formats = settings.DATE_INPUT_FORMATS


class BaseDateTimeField(BaseCustomField):
    """
    DateTimeField
    """
    field_type = _("Data e Ora")
    is_complex = True

    def __init__(self, *args, **data_kwargs):
        # Data DateField
        self.data = BaseDateField(*args, **data_kwargs)
        self.data.label = _("{} (Data)").format(data_kwargs.get('label'))
        self.data.name = "{}_dyn".format(format_field_name(self.data.label))
        self.data.parent = self

        # Ore SelectBox
        self.hour = CustomSelectBoxField(*args, **data_kwargs)
        self.hour.label = _("{} (Ore)").format(data_kwargs.get('label'))
        self.hour.name = "{}_dyn".format(format_field_name(self.hour.label))
        self.hour.choices = [(i,i) for i in range(24)]
        self.hour.initial = 0
        self.hour.parent = self

        # Minuti SelectBox
        self.minute = CustomSelectBoxField(*args, **data_kwargs)
        self.minute.label = _("{} (Minuti)").format(data_kwargs.get('label'))
        self.minute.name = "{}_dyn".format(format_field_name(self.minute.label))
        self.minute.choices = [(i,i) for i in range(60)]
        self.minute.initial = 0
        self.minute.parent = self

    def get_fields(self):
        return [self.data, self.hour, self.minute]


class DateStartEndComplexField(BaseCustomField):
    """
    Field composto da DataInizio (DateField) e DataFine (DateField)
    """
    field_type = _("Data inizio e Data fine")
    is_complex = True

    def __init__(self, *args, **data_kwargs):
        parent_label = data_kwargs.get('label')

        # Data Inizio
        self.start = BaseDateField(*args, **data_kwargs)
        self.start.required = data_kwargs.get('required')
        self.start.label = _("{} (Data inizio)").format(parent_label)
        self.start.name = "{}_dyn".format(format_field_name(self.start.label))

        # Riferimento a DateStartEndComplexField
        self.start.parent = self

        # Data Fine
        self.end = BaseDateField(*args, **data_kwargs)
        self.end.required = data_kwargs.get('required')
        self.end.label = _("{} (Data fine)").format(parent_label)
        self.end.name = "{}_dyn".format(format_field_name(self.end.label))

        # Riferimento a DateStartEndComplexField
        self.end.parent = self

    def get_fields(self):
        fields = [self.start, self.end]
        return fields

    def raise_error(self, name, cleaned_data, **kwargs):
        """
        Essendo un campo complesso che non ha riferimenti ai vincoli
        imposti dal bando, si eseguono solo i controlli standard sulle
        date di inizio e di fine
        """
        errors = []
        start_value = cleaned_data.get(self.start.name)
        end_value = cleaned_data.get(self.end.name)

        # Se il campo non viene correttamente inizializzato
        if not cleaned_data.get(name): return []

        # Si valuta 'Data Inizio'
        if name == self.start.name:
            # Se data_inizio > data_fine
            if end_value and start_value > end_value:
                errors.append(_("La data di inizio non può "
                                "essere successiva a quella di fine"))

        return errors


class ProtocolloField(BaseCustomField):
    """
    Tipo,Numero e Data protocollo (o altro tipo di numerazione)
    """
    field_type = "Protocollo (tipo/numero/data)"
    is_complex = True

    def __init__(self, *args, **data_kwargs):
        parent_label = data_kwargs.get('label')

        # Tipo protocollo. SelectBox
        self.tipo = CustomSelectBoxField(*args, **data_kwargs)
        self.tipo.required = data_kwargs.get('required')
        self.tipo.label = _("{} (Tipo numerazione)").format(parent_label)
        self.tipo.name = "{}_dyn".format(format_field_name(self.tipo.label))
        self.tipo.help_text = _("Scegli se protocollo/decreto/delibera, "
                                "al/alla quale la numerazione è riferita")
        self.tipo.choices += [(i[0].lower().replace(' ', '_'), i[1]) \
                             for i in CLASSIFICATION_LIST]
        self.tipo.parent = self

        # Numero protocollo. CharField
        self.numero = CustomCharField(*args, **data_kwargs)
        self.numero.required = data_kwargs.get('required')
        self.numero.label = _("{} (Numero Protocollo/Delibera/Decreto)").format(parent_label)
        self.numero.name = "{}_dyn".format(format_field_name(self.numero.label))
        self.numero.help_text = _("Indica il numero del "
                                  "protocollo/decreto/delibera")
        self.numero.parent = self

        # Data protocollo. DateField
        self.data = BaseDateField(*args, **data_kwargs)
        self.data.required = data_kwargs.get('required')
        self.data.label = _("{} (Data Protocollo/Delibera/Decreto)").format(parent_label)
        self.data.name = "{}_dyn".format(format_field_name(self.data.label))
        self.data.help_text = _("Indica la data del protocollo/delibera/decreto")
        self.data.parent = self

    def get_fields(self):
        return [self.tipo, self.numero, self.data]

    def get_data_name(self):
        return self.data.name

    def raise_error(self, name, cleaned_data, **kwargs):
        """
        Questo campo complesso subisce controlli inerenti i parametri
        imposti dal bando e allo stesso tempo si relaziona, se presente,
        a DataInizio e DataFine in range
        """
        value = cleaned_data.get(name)

        if not value: return [_("Valore mancante")]

        # Si valuta 'Data protocollo'
        if name == self.data.name:
            errors = []
            # Se la data è successiva ad oggi
            if _successivo_ad_oggi(value):
                errors.append(_("La data di protocollo non può essere"
                                " successiva ad oggi"))
            return errors


class CustomHiddenField(CharField, BaseCustomField):
    """
    CharField Hidden
    """
    field_type = _("Campo nascosto")

    def define_value(self, custom_value, **kwargs):
        self.widget = forms.HiddenInput(attrs={'value': custom_value})


class CustomComplexTableField(ChoiceField, BaseCustomField):
    """
    CustomComplexTableField
    """
    validation_error = _("Questo campo necessita di almeno una riga")
    field_type = _("Inserimenti multipli")
    is_complex = True
    is_formset = True
    choices = None

    def get_formset(self):
        if not self.choices: return
        return build_formset(choices=self.choices)

    def clean(self, *args, **kwargs):
        # Se non ci sono forms e il campo è required allora deve fare
        # l'override. In caso contrario, deve essere ignorato e il
        # clean dei campi viene delegato ai singoli form
        if self.required and not self.widget.formset.forms:
            raise ValidationError(self.validation_error)
        return

    def define_value(self, custom_value, **kwargs):
        """
        Se presenti, sostituisce alle opzioni di default
        quelle di 'custom_value'
        """
        if custom_value:
            elements = _split_choices_in_list_canc(custom_value)
            self.choices = elements



from django.utils.translation import ugettext_lazy as _


CLASSIFICATION_LIST = (('protocollo', _('Protocollo')),
                       ('decreto_rettorale', _('Decreto Rettorale (D.R.)')),
                       ('decreto_direttore_generale', _('Decreto del Direttore Generale (D.D.G.)')),
                       ('decreto_dirigente_struttura', _('Decreto del Direttore Dipartimento o Dirigente Struttura')),
                       ('decreto_direttore_cr', _('Decreto del Direttore del Centro Residenziale (D.CR.)')),
                       ('decreto_prorettore', _('Decreto del Prorettore (Centro Residenziale)')),
                       ('delibera_dipartimento_facolta', _('Delibera di Dipartimento/Facoltà')),
                       ('delibera_senato', _('Delibera del Senato')),
                       ('delibera_cda', _('Delibera del C.D.A.')))

# 2.5MB - 2621440
# 5MB - 5242880
# 10MB - 10485760
# 20MB - 20971520
# 50MB - 5242880
# 100MB 104857600
# 250MB - 214958080
# 500MB - 429916160
MAX_UPLOAD_SIZE = 10485760
IMG_PERMITTED_UPLOAD_FILETYPE = ("image/jpeg", "image/png",
                                 "image/gif", "image/x-ms-bmp")
BASE_FILETYPE = ('application/pdf','application/pkcs7-mime')
PERMITTED_UPLOAD_FILETYPE = BASE_FILETYPE + IMG_PERMITTED_UPLOAD_FILETYPE

# maximum permitted filename lengh in attachments, uploads
ATTACH_NAME_MAX_LEN = 50

ATTACHMENTS_DICT_PREFIX = "allegati"

# attachments validation messages
WRONG_TYPE = _("Per favore esegui l'upload di soli file "
               "in {}. "
               "Attualmente questo è '{}'")
WRONG_SIZE = _("Per favore mantieni la dimensione del file entro {}. "
               "Attualmente questo è {}")
WRONG_LENGTH = _("Per favore usa una lunghezza massima del nome dell'allegato "
                 "inferiore a {}. Attualmente hai inserito un nome di {} caratteri")

# formset special words
FORMSET_TEMPLATE_NAMEID = 'NNNNN'
MANAGEMENT_FORMSET_STRINGS = [FORMSET_TEMPLATE_NAMEID,
                              '-TOTAL_FORMS', '-INITIAL_FORMS',
                              '-MAX_NUM_FORMS','-MIN_NUM_FORMS']



import re

from django import forms
from django.template.loader import render_to_string
from django.utils.safestring import mark_safe

from . dynamic_fields import build_formset, CustomCharField
from . settings import FORMSET_TEMPLATE_NAMEID


class FormsetdWidget(forms.Widget):

    def __init__(self, *attrs, **kwargs):
        self.template = 'widgets/formset.html'
        self.readonly = False
        init_data_list = ['choices', 'data']
        field_required = kwargs.pop('field_required')
        self.prefix = kwargs.pop('prefix')
        for i in init_data_list:
            if i in kwargs.keys():
                setattr(self, i, kwargs.pop(i))

        if getattr(self, 'data'):
            # It MUST be a list!
            # self.data = [{'ccc': 'dfgdfgdfg', 'co': 'sono', 'col1': 'un', 'data': '2019-04-03'},
            #              {'ccc': '234234', 'co': 'son44o', 'col1': 'un333', 'data': '2016-04-03'}]
            self.formset = build_formset(choices=self.choices,
                                         required=field_required,
                                         prefix=self.prefix,
                                         data=self.data)
            self.formset.is_valid()
        else:
            # this initialized the formset as void
            self.formset = build_formset(choices=self.choices,
                                         required=field_required,
                                         extra=0,
                                         prefix=self.prefix)
        super().__init__(*attrs, **kwargs)

    def get_js_template(self):
        formset = build_formset(choices=self.choices,
                                prefix=self.prefix,
                                extra=1)
        res = re.sub('{}-0'.format(self.prefix),
                     '{}-{}'.format(self.prefix,
                                    FORMSET_TEMPLATE_NAMEID),
                     formset.forms[0].as_table())
        return mark_safe(res)

    def render(self, name='', value='', attrs=None, renderer=None):
        context_data = {'formset_id': self.prefix,
                        'template_generic_id': FORMSET_TEMPLATE_NAMEID,
                        'formset': self.formset,
                        'formset_template': self.get_js_template(),
                        'readonly': self.readonly,}
        html_output = render_to_string(self.template, context_data)
        return mark_safe(html_output)

    def make_readonly(self, attr='disabled'):
        self.readonly = True
        for form in self.formset:
            for generic_field in form:
                field = form.fields[generic_field.name]
                widget = field.widget
                widget.attrs[attr] = True
        return True



import json

from collections import OrderedDict
from django.db import models

from . dynamic_fields import get_fields_types
from . forms import BaseDynamicForm
from . utils import get_as_dict

class DynamicFieldMap(models.Model):
    """
    """
    name = models.CharField(max_length=150,)
    field_type = models.CharField(max_length=100,
                                  choices = get_fields_types())
    valore = models.CharField(max_length=255,
                              blank=True,
                              default='',
                              verbose_name='Lista di Valori',
                              help_text="Viene considerato solo se si sceglie"
                                        " 'Menu a tendina' oppure 'Serie di Opzioni'."
                                        " (Es: valore1;valore2;valore3...)")
    is_required = models.BooleanField(default=True)
    aiuto = models.CharField(max_length=254, blank=True, default='')
    ordinamento = models.PositiveIntegerField(help_text="posizione nell'ordinamento",
                                              blank=True,
                                              default=0)

    @staticmethod
    def build_constructor_dict(fields):
        """
        costruisco il dizionario da dare in pasto
        al costruttore del form dinamico
        """
        constructor_dict = OrderedDict()
        for field in fields:
            d = {'label': field.name,
                 'required' : field.is_required,
                 'help_text' : field.aiuto}
            constructor_dict[field.name] = (field.field_type,
                                            d, field.valore)
        return constructor_dict

    @staticmethod
    def get_form(class_obj=BaseDynamicForm,
                 constructor_dict={},
                 custom_params={},
                 remove_filefields=False,
                 remove_datafields=False,
                 fields_order=[],
                 *args, **kwargs):
        if class_obj == BaseDynamicForm:
            custom_params = {}
        form = class_obj(constructor_dict=constructor_dict,
                         custom_params=custom_params,
                         *args, **kwargs)

        if remove_filefields:
            form.remove_files(allegati = remove_filefields)
        if remove_datafields:
            form.remove_datafields()
        if fields_order:
            form.order_fields(fields_order)
        return form

    class Meta:
        abstract = True
        ordering = ('ordinamento',)


class SavedFormContent(models.Model):
    """
    """
    # libreria esterna oppure cambio client per JsonField
    # non serve gestirlo come JsonField perchè non vi facciamo ricerche al suo interno ;)
    modulo_compilato = models.TextField()

    @staticmethod
    def compiled_form(data_source=None,
                      constructor_dict={},
                      files=None,
                      remove_filefields=True,
                      remove_datafields=False,
                      form_source=None,
                      # fields_order=[],
                      extra_datas={},
                      **kwargs):
        json_dict = json.loads(data_source)
        data = get_as_dict(json_dict, allegati=False)
        if extra_datas:
            for k,v in extra_datas.items():
                data[k]=v
        if not form_source:
            form_source = DynamicFieldMap
        form = form_source.get_form(constructor_dict=constructor_dict,
                                    data=data,
                                    files=files,
                                    remove_filefields=remove_filefields,
                                    remove_datafields=remove_datafields,
                                    **kwargs)
        # Già invocato nel "form_source", ma è bene tenerlo come riferimento
        # if fields_order:
            # form.order_fields(fields_order)
        return form

    @staticmethod
    def compiled_form_readonly(form, attr='disabled', fields_to_remove=[]):
        """
        Restituisce una versione "più pulita" di compiled_form
        - I field non compilati non vengono mostrati (remove_not_compiled_fields())
        - Il campo "titolo" di default non viene mostrato
        - I fields sono readonly
        NOTA: i field select non risentono dell'attributo readonly!!!
        Usato nei metodi che producono le anteprime non modificabili
        dei moduli compilati
        """
        form.remove_not_compiled_fields()
        for field_to_remove in fields_to_remove:
            del form.fields[field_to_remove]
        for generic_field in form:
            field = form.fields[generic_field.name]
            widget = field.widget
            widget.attrs[attr] = True
            # If field is a Formset, the widget will make it readonly
            if field.is_formset:
                widget.make_readonly(attr)
                continue
            # Es: TextArea non ha attributo 'input_type'
            # Senza questo controllo il codice genera un'eccezione
            if not hasattr(widget, 'input_type'):
                # widget.attrs[attr] = True
                continue
            tipo = widget.input_type
            if tipo in ['select', 'checkbox', 'radiobox']:
                widget.attrs['disabled'] = True
            # else:
                # widget.attrs[attr] = True
        return form

    class Meta:
        abstract = True



import logging
import os

from django_form_builder import dynamic_fields
from django_form_builder.settings import ATTACHMENTS_DICT_PREFIX, CLASSIFICATION_LIST
from django_form_builder.templatetags.django_form_builder_tags import *
from django_form_builder.utils import get_labeled_errors

from . base import BaseTest


logger = logging.getLogger('my_logger')

class TestInvalidFields(BaseTest):

    def test_integer_valid(self):
        """
        Test valid value in PositiveIntegerField
        """
        form = self.single_field_form('PositiveIntegerField', 44)
        logger.info("Test valid PositiveIntegerField")
        assert form.is_valid()

    def test_integer_invalid(self):
        """
        Test invalid value in PositiveIntegerField
        Decimal value instead of Integer
        """
        form = self.single_field_form('PositiveIntegerField', 16e50)
        logger.info("Test invalid PositiveIntegerField")
        logger.info(get_labeled_errors(form))
        self.assertFalse(form.is_valid())

    def test_float_valid(self):
        """
        Test valid value in PositiveFloatField
        """
        form = self.single_field_form('PositiveFloatField', 4)
        logger.info("Test valid PositiveFloatField")
        assert form.is_valid()

    def test_float_invalid(self):
        """
        Test invalid value in PositiveFloatField
        """
        form = self.single_field_form('PositiveFloatField', 16e50)
        logger.info("Test invalid PositiveFloatField")
        logger.info(get_labeled_errors(form))
        self.assertFalse(form.is_valid())

    def test_float_invalid_2(self):
        """
        Test invalid value in PositiveFloatField
        """
        form = self.single_field_form('PositiveFloatField', 4.3333)
        logger.info("Test invalid PositiveFloatField")
        logger.info(get_labeled_errors(form))
        self.assertFalse(form.is_valid())

    def test_date_startend_valid(self):
        """
        Test valid values in DateStartEndComplexField
        """
        data = {'data_inizio_dyn': '12/01/2020',
                'data_fine_dyn': '12/03/2020'}
        form = self.complex_field_form('DateStartEndComplexField', data)
        logger.info("Test valid DateStartEndComplexField")
        assert form.is_valid()

    def test_date_startend_invalid(self):
        """
        Test invalid values in DateStartEndComplexField
        Start date > End date
        """
        data = {'data_inizio_dyn': '12/01/2022',
                'data_fine_dyn': '12/03/2020'}
        form = self.complex_field_form('DateStartEndComplexField', data)
        logger.info("Test valid DateStartEndComplexField")
        self.assertFalse(form.is_valid())

    def test_datetimefield_valid(self):
        """
        Test valid values in BaseDateTimeField
        """
        super_label = 'Field label'
        field_id = dynamic_fields.format_field_name(super_label)
        data = {'{}_data_dyn'.format(field_id): '11/01/2020',
                '{}_ore_dyn'.format(field_id): '12',
                '{}_minuti_dyn'.format(field_id): '12'}
        form = self.complex_field_form('BaseDateTimeField',
                                        data,
                                        super_label)
        logger.info("Test valid BaseDateTimeField")
        assert form.is_valid()

    def test_protocollo_invalid(self):
        """
        Test invalid values in ProtocolloField
        data_protocollo > current date
        """
        data = {'tipo_numerazione_dyn': CLASSIFICATION_LIST[0][0],
                'numero_protocollo_dyn': '12312 text',
                'data_protocollo_dyn': '12/03/2010'}
        form = self.complex_field_form('ProtocolloField', data)
        logger.info("Test valid ProtocolloField")
        assert form.is_valid()

    def test_filefield(self):
        """
        Test valid value in PositiveIntegerField
        """
        file_test = self.create_fake_file()
        form = self.single_field_form('CustomFileField',
                                      file_value=file_test)
        logger.info("Test valid CustomFileField")
        assert form.is_valid()

    def test_signed_pdf(self):
        """
        Test valid value in PositiveIntegerField
        """
        name = 'test_signed.pdf'
        file_obj = self.get_file(name)
        form = self.single_field_form('CustomSignedPdfField',
                                      file_value=file_obj)
        logger.info("Test valid CustomSignedPdfField")
        assert form.is_valid()

    def test_signed_pdf_show_details(self):
        """
        Test valid value in PositiveIntegerField
        """
        name = 'test_signed.pdf'
        file_obj = self.get_file(name)
        field = self.create_field('CustomSignedPdfField')[0]
        field_id = dynamic_fields.format_field_name(field.label)
        initial_fields = {field_id: field}
        files = {field_id: file_obj}
        form = self.get_baseform(initial_fields=initial_fields,
                                 files=files)
        logger.info("Test valid PDF signed file (sign details)")
        logger.info(get_attachment_sign_details(form,
                                                self.get_filepath(),
                                                field_id,
                                                name))
        assert form.is_valid()

    def test_signed_p7m(self):
        """
        Test valid value in PositiveIntegerField
        """
        name = 'test_signed.pdf.p7m'
        file_obj = self.get_file(name)
        form = self.single_field_form('CustomSignedP7MField', file_value=file_obj)
        logger.info("Test valid CustomSignedP7MField")
        assert form.is_valid()
